/**

 Implementation DrawingBoard class
 Fraga 10/09/2000 

**/

#include "board.h"
#include <qpainter.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>


DrawingBoard::DrawingBoard( QWidget *parent, const char *name )
        : QWidget( parent, name )
{
    ang = 45;
    setPalette( QPalette( QColor( 250, 250, 200) ) );
	setMouseTracking( TRUE );
	px = py = 0;
}


void DrawingBoard::setAngle( int degrees )
{
    if ( degrees < 5 )
	degrees = 5;
    if ( degrees > 70 )
	degrees = 70;
    if ( ang == degrees )
	return;
    ang = degrees;
    repaint();
//    emit angleChanged( ang );
}


void DrawingBoard::paintEvent( QPaintEvent * )
{
    QString s = "Angle = " + QString::number( ang );
    QPainter p( this );
    p.drawText( 200, 200, s );

	testLines( &p );
}

void DrawingBoard::mouseMoveEvent( QMouseEvent * e )
{
	QPoint c = e->pos( );  

	px = c.x( );
	py = c.y( );
	if ( px>=0 && py>=0 )  emit changePos ( );
}

void DrawingBoard::resizeEvent ( QResizeEvent * )
{
	w = width( );
	h = height( );
}

QSizePolicy DrawingBoard::sizePolicy() const
{
    return QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
}

/** Drawing primitives **/
void DrawingBoard::MidPointLine( int x1, int y1, int x2, int y2, QPainter *p )
{
	int tmp;
	int dx, dy;
	int d, incrE, incrNE;
	int x, y, flag=0, algo=0;

	if ( y2 < y1 ) {  /** Swap points **/
		tmp = y1; y1 = y2; y2 = tmp;
		tmp = x1; x1 = x2; x2 = tmp;
	}

	dx = abs(x2 - x1);
	dy = y2 - y1;

	if ( dx == 0 ) {
		if ( dy == 0 ) return;

		for( y=y1; y<=y2; y++ )
        	p->drawPoint( x1, y ); 
	}
	else if ( dy == 0 ) {
		if ( x1 > x2 ) { /** swap x's **/
			tmp = x1; x1 = x2; x2 = tmp;
		}
		for( x=x1; x<=x2; x++ )
        	p->drawPoint( x, y1 ); 
	}
	else if ( dx == dy ) {
		if ( x2 > x1 ) {  /** Normal algorithm **/
			for( x=x1, y=y1; x<=x2; x++, y++ )
       		 	p->drawPoint( x, y ); 
		}
		else {
			for( x=x1, y=y1; x>=x2; x--, y++ )
       		 	p->drawPoint( x, y ); 
		}
	}
	else {
		if ( x2 < x1 ){
			algo++; /** Algorithm with x's decreasing **/ 
		}

		if ( dy > dx ) {  /** swap x's by y's and viceversa **/
			tmp = dx; dx = dy; dy = tmp;	
			tmp = x1; x1 = y1; y1 = tmp;
			tmp = x2; x2 = y2; y2 = tmp;
			flag++;
		}
		d = 2*dy + dy;
		incrE = 2*dy;
		incrNE = 2*(dy - dx);

		x = x1;
		y = y1; 
       	if ( !flag) p->drawPoint( x, y ); 
       	else p->drawPoint( y, x ); 

		if ( !algo ) {  /** Normal algoritm **/
			while ( x < x2 ) {
				if ( d <= 0 ) {
					x++;
					d += incrE;
				}
				else {
					x++;
					y++;
					d += incrNE;
				} 
       			if ( !flag ) p->drawPoint( x, y ); 
       			else p->drawPoint( y, x ); 
			}
		}
		else {
			if ( !flag ) {
				while ( x > x2 ) {
					if ( d <= 0 ) {
						x--;
						d += incrE;
					}
					else {
						x--;
						y++;
						d += incrNE;
					} 
       				p->drawPoint( x, y ); 
				}
			}
			else {
				while ( x < x2 ) {
					if ( d <= 0 ) {
						x++;
						d += incrE;
					}
					else {
						x++;
						y--;
						d += incrNE;
					} 
       				p->drawPoint( y, x ); 
				}
			}
		}
	}
}

void DrawingBoard::testLines( QPainter *p )
{
	int cx = w/2;
	int cy = h/2;
	int incrang = 5;
	int l = 150;

	int n = 360/incrang;

	for ( int i=0; i<n; i++ ) {
	// for ( int i=18; i<=27; i++ ) {
		float ang = i*incrang;

		ang = ang * ANGTORAD;

		float val = l*cos(ang) + cx;
		int px = Round( val );
		
		val = l*sin(ang) + cy;
		int py = Round( val );

		MidPointLine( cx, cy, px, py, p );
	}
}

